---------------------------------------------------------------------
--== Import ===--
---------------------------------------------------------------------

import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "Souru.LootLogs.Type"
import "Souru.LootLogs.Class"
import "Souru.LootLogs.UI"

---------------------------------------------------------------------
--==  ===--
---------------------------------------------------------------------

me = Turbine.Gameplay.LocalPlayer:GetInstance()
myName = me:GetName()

window = nil
icon = nil

---------------------------------------------------------------------
--== localization ===--
---------------------------------------------------------------------

local locale = "en"
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	import "Souru.LootLogs.Localization.de"
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	import "Souru.LootLogs.Localization.fr"
else
	import "Souru.LootLogs.Localization.en"
end

---------------------------------------------------------------------
--== Basic Functions ===--
---------------------------------------------------------------------

function SaveLogs()

	LootList["locale"] = locale
	if locale == "en" then
		Turbine.PluginData.Save(Turbine.DataScope.Account, "LootLogsNew", LootList)
	else
		Turbine.PluginData.Save(Turbine.DataScope.Account, "LootLogsNew", convertToEuro(LootList))
	end

end

function SaveSettings()

	Turbine.PluginData.Save(Turbine.DataScope.Character, "LootSettingsNew", LootSettings)

end

function CheckForMe()

	for bossindex,v in ipairs(LootList) do 
		
		if v["name"] == myName then
			return bossindex
		end
		
	end
	
	return nil

end

function AddMe()

	local index = table.getn(LootList)+1
	
	LootList[index] = {}
	LootList[index]["name"] = myName
	LootList[index]["loot"] = {}
	
	currentIndex = index
	
	SaveLogs()

end

function CheckForReset()

	local gt = Turbine.Engine.GetGameTime()

	for pIndex, player in ipairs(LootList) do

		local contentCounter = 0

		for cIndex, content in ipairs(_G.Instances) do

			if player["loot"][cIndex] ~= nil then

				local iniCounter = 0

				for iIndex, ini in ipairs(content["instances"]) do

					if player["loot"][cIndex][iIndex] ~= nil then

						local tierCounter = 0

						for tIndex, tier in ipairs(ini) do

							if player["loot"][cIndex][iIndex][tIndex] ~= nil then

								local bossCounter = 0

								for bIndex, boss in ipairs(tier) do

									if player["loot"][cIndex][iIndex][tIndex][bIndex] ~= nil then

										if player["loot"][cIndex][iIndex][tIndex][bIndex]["reset"] <= gt then

											player["loot"][cIndex][iIndex][tIndex][bIndex] = nil

										else 

											contentCounter = contentCounter + 1
											iniCounter = iniCounter + 1
											tierCounter = tierCounter + 1
											bossCounter = bossCounter + 1

										end
		
									end
		
								end

								if bossCounter == 0 then

									player["loot"][cIndex][iIndex][tIndex] = nil

								end

							end

						end

						if tierCounter == 0 then

							player["loot"][cIndex][iIndex] = nil

						end

					end

				end

				if iniCounter == 0 then

					player["loot"][cIndex] = nil

				end

			end

		end

		if contentCounter == 0 then

			player["loot"] = {}

		end

	end

	SaveLogs()

end

function ResetToString(resetTime)

	local reset = (resetTime-Turbine.Engine.GetGameTime())/3600
	local returnString = math.floor(reset/24)
	reset = reset-(returnString*24)
	returnString = returnString.."d - "..math.floor(reset).."h"

	return returnString

end

function ClearSaveFile()

	LootList = nil
	LootList = {}
	AddMe()
	SaveLogs()

end

function convertToEuro(dataRaw)
	local newData = {};
	for i, myData in pairs(dataRaw) do
		local tempIndex = nil;
		local tempData = nil;
		if (type(i) == "number") then
			tempIndex = tostring(i);
		else
			tempIndex = i;
		end
		if (type(myData) == "table") then
			tempData = convertToEuro(myData);
		elseif (type(myData) == "number") then
			tempData = tostring(myData);
		else
			tempData = myData;
		end
		newData[tempIndex] = tempData;
	end
	return newData;
end

function convertFromEuro(dataRaw)
	local newData = {}
	for i, myData in pairs(dataRaw) do
		local tempIndex = tonumber(i);
		if (tempIndex == nil) then
			tempIndex = i;
		end
		local tempData = nil;

		if (type(myData) == "table") then
			tempData = convertFromEuro(myData);
		else
			tempData = tonumber(myData);
			if (tempData == nil) then
				tempData = myData;
			end
		end
		newData[tempIndex] = tempData;
	end
	return newData;
	
end

---------------------------------------------------------------------
--== UI Functions ===--
---------------------------------------------------------------------

function OpenSettings()
	if  window == nil then
		CheckForReset()
		window = UI.Window()
	else
		window:Close()
	end
end

function ShowBossName(bool, text, left, top)

	if  window ~= nil then
		window:ShowBossName(bool, text, left, top)
	end
end

function LootUpdatedSettings()
	if  window ~= nil then
		window:LootUpdatedSettings()
	end
end

function CloseSettings()
	window = nil
end

---------------------------------------------------------------------
--== Load ===--
---------------------------------------------------------------------

locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
end

if Turbine.PluginData.Load(Turbine.DataScope.Account, "LootLogsNew") ~= nil then

	LootList = Turbine.PluginData.Load(Turbine.DataScope.Account, "LootLogsNew")
	if LootList["locale"] ~= "en" and LootList["locale"] ~= nil then
		LootList = convertFromEuro(LootList);
	end

	currentIndex = CheckForMe()
	
	if currentIndex == nil then
		AddMe()
	end
	
else

	LootList = {}

	AddMe()

	SaveLogs()

end

if Turbine.PluginData.Load(Turbine.DataScope.Character, "LootSettingsNew") ~= nil then

	LootSettings = Turbine.PluginData.Load(Turbine.DataScope.Character, "LootSettingsNew")

	if LootSettings["show_notifications"] == nil then
		LootSettings["show_notifications"] = true
	end

else

	LootSettings = {}

	LootSettings["window"] = {}
	LootSettings["window"]["left"] = 500
	LootSettings["window"]["top"] = 500
	LootSettings["window"]["width"] = 850
	LootSettings["window"]["height"] = 500
	LootSettings["icon"] = {}
	LootSettings["icon"]["left"] = 200
	LootSettings["icon"]["top"] = 200

end

---------------------------------------------------------------------
--== Icon ===--
---------------------------------------------------------------------

icon = UI.WindowIcon()

---------------------------------------------------------------------
--== Loot Functions ===--
---------------------------------------------------------------------

function CalcNextReset(instanceObject)

	local date = Turbine.Engine.GetDate()
	local nextResetDay = nil
	local timeUntilReset = 60-date["Minute"]

	if(date["Hour"] <= 8) then

		timeUntilReset = timeUntilReset + ((8 - date["Hour"])*60)
		nextResetDay = date["DayOfWeek"]

	else

		timeUntilReset = timeUntilReset + ((8 + 24 - date["Hour"])*60)
		nextResetDay = (date["DayOfWeek"] + 1) % 8

		if nextResetDay == 0 then
			nextResetDay = 1
		end

	end

	local counter = 0
	while (instanceObject["reset"][nextResetDay] == false)do

		counter = counter + 1
		nextResetDay = (nextResetDay + 1) % 8

		if nextResetDay == 0 then
			nextResetDay = 1
		end

	end

	timeUntilReset = timeUntilReset + (counter * 24 * 60)

	return ((timeUntilReset * 60) + Turbine.Engine.GetGameTime())
end

function FindChest(message, value)

	for contentIndex, content in ipairs(_G.Instances) do

		for instanceIndex, instance in ipairs(content["instances"]) do

			for tierIndex, tier in ipairs(instance) do

				for bossIndex, chest in ipairs(tier) do

					if string.find(message, chest["text"]) then

						if LootList[currentIndex]["loot"][contentIndex] == nil then
							LootList[currentIndex]["loot"][contentIndex] = {}
						end

						if LootList[currentIndex]["loot"][contentIndex][instanceIndex] == nil then
							LootList[currentIndex]["loot"][contentIndex][instanceIndex] = {}
						end

						if LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex] == nil then
							LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex] = {}
						end

						if LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex][bossIndex] == nil then
							LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex][bossIndex] = {}
						end

						LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex][bossIndex]["value"] = value
						LootList[currentIndex]["loot"][contentIndex][instanceIndex][tierIndex][bossIndex]["reset"] = CalcNextReset(_G.Instances[contentIndex]["instances"][instanceIndex][tierIndex])
						LootUpdatedSettings()
						SaveLogs()

						local notification_text = "<rgb=#ffa500>"..instance["name"].."</rgb> <rgb=#FF0000>"..tier["name"].."</rgb> has been Updated."
						icon:AddNotification(notification_text)

					end

				end

			end

		end

	end

end

function Turbine.Chat.Received(sender, args)

	if args.ChatType == 21 or args.ChatType == Turbine.ChatType.Standard then

		if (string.find(args.Message, _G.Ini_FavoredCompletions)) ~= nil then -- FAVORED

			FindChest(string.sub(args.Message, 0, string.find(args.Message, _G.Ini_FavoredCompletions)-1), tonumber(string.match(string.sub(args.Message, string.find(args.Message,  _G.Ini_FavoredCompletions)) , "%d+")))

		elseif (string.find(args.Message, _G.Ini_Completions)) ~= nil then -- COMON
		
			FindChest(string.sub(args.Message, 0, string.find(args.Message, _G.Ini_Completions)-1), tonumber(string.match(string.sub(args.Message, string.find(args.Message, _G.Ini_Completions)) , "%d+"))*(-1))

		elseif (string.find(args.Message, _G.Ini_ResetIn)) ~= nil then -- RESET

			FindChest(string.sub(args.Message, 0, string.find(args.Message, _G.Ini_ResetIn)-1), 0)

		elseif (string.find(args.Message,_G.Quest_Completed)) ~= nil then -- QUESTS

			FindChest(string.sub(args.Message, 0, string.find(args.Message, _G.Quest_Completed)-1), 0)

		end

	end

end
